import 'package:flutter/material.dart';
import 'package:flutterskel/resources/colors/ColorResources.dart';
import 'package:flutterskel/resources/dimensions/DimensionResources.dart';

/// Base class for fonts
/// Provides basic Material Design textStyles
///
/// The 2018 spec has thirteen text styles:
/// ```
/// NAME         SIZE  WEIGHT  SPACING
/// headline1    96.0  light   -1.5
/// headline2    60.0  light   -0.5
/// headline3    48.0  normal   0.0
/// headline4    34.0  normal   0.25
/// headline5    24.0  normal   0.0
/// headline6    20.0  medium   0.15
/// subtitle1    16.0  normal   0.15
/// subtitle2    14.0  medium   0.1
/// body1        16.0  normal   0.5
/// body2        14.0  normal   0.25
/// button       14.0  medium   0.75
/// caption      12.0  normal   0.4
/// overline     10.0  normal   1.5
///
/// ```
/// Where "light" is `FontWeight.w300`, "normal" is `FontWeight.w400` and
/// "medium" is `FontWeight.w500`.
abstract class FontResources {
  // Colors
  ColorResources colors;

  // Dimensions
  DimensionResources dimensions;

  // Font Family
  String fontFamily;

  // Text Styles
  TextStyle _baseStyle;
  TextStyle _headline1, _headline2, _headline3, _headline4, _headline5, _headline6;
  TextStyle _subtitle1, _subtitle2;
  TextStyle _body1, _body2;
  TextStyle _button, _caption, _overline;

  FontResources({
    @required this.colors,
    @required this.dimensions,
    TextStyle baseTextStyle,
    this.fontFamily,
  }) {
    _baseStyle = baseTextStyle != null ? baseTextStyle : (fontFamily != null ? TextStyle(fontFamily: fontFamily) : TextStyle());
  }

  void setColors(ColorResources colorResources) {
    this.colors = colorResources;
    clearCache();
  }

  void setDimensions(DimensionResources dimensionResources) {
    this.dimensions = dimensionResources;
    clearCache();
  }

  @mustCallSuper
  void clearCache() {
    _headline1 = null;
    _headline2 = null;
    _headline3 = null;
    _headline4 = null;
    _headline5 = null;
    _headline6 = null;
    _subtitle1 = null;
    _subtitle2 = null;
    _body1 = null;
    _body2 = null;
    _button = null;
    _caption = null;
    _overline = null;
  }
  
  TextStyle get headline1 => _headline1 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline1,
    height: dimensions.fontHeightHeadline1,
    fontWeight: FontWeight.w300,
    letterSpacing: dimensions.fontLetterSpacingHeadline1,
    wordSpacing: dimensions.fontWordSpacingHeadline1,
  );

  TextStyle get headline2 => _headline2 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline2,
    height: dimensions.fontHeightHeadline2,
    fontWeight: FontWeight.w300,
    letterSpacing: dimensions.fontLetterSpacingHeadline2,
    wordSpacing: dimensions.fontWordSpacingHeadline2,
  );

  TextStyle get headline3 => _headline3 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline3,
    height: dimensions.fontHeightHeadline3,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingHeadline3,
    wordSpacing: dimensions.fontWordSpacingHeadline3,
  );

  TextStyle get headline4 => _headline4 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline4,
    height: dimensions.fontHeightHeadline4,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingHeadline4,
    wordSpacing: dimensions.fontWordSpacingHeadline4,
  );

  TextStyle get headline5 => _headline5 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline5,
    height: dimensions.fontHeightHeadline5,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingHeadline5,
    wordSpacing: dimensions.fontWordSpacingHeadline5,
  );

  TextStyle get headline6 => _headline6 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeHeadline6,
    height: dimensions.fontHeightHeadline6,
    fontWeight: FontWeight.w500,
    letterSpacing: dimensions.fontLetterSpacingHeadline6,
    wordSpacing: dimensions.fontWordSpacingHeadline6,
  );

  TextStyle get subtitle1 => _subtitle1 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeSubtitle1,
    height: dimensions.fontHeightSubtitle1,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingSubtitle1,
    wordSpacing: dimensions.fontWordSpacingSubtitle1,
  );

  TextStyle get subtitle2 => _subtitle2 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeSubtitle2,
    height: dimensions.fontHeightSubtitle2,
    fontWeight: FontWeight.w500,
    letterSpacing: dimensions.fontLetterSpacingSubtitle2,
    wordSpacing: dimensions.fontWordSpacingSubtitle2,
  );

  TextStyle get body1 => _body1 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeBody1,
    height: dimensions.fontHeightBody1,
    fontWeight: FontWeight.w300,
    letterSpacing: dimensions.fontLetterSpacingBody1,
    wordSpacing: dimensions.fontWordSpacingBody1,
  );

  TextStyle get body2 => _body2 ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeBody2,
    height: dimensions.fontHeightBody2,
    fontWeight: FontWeight.w300,
    letterSpacing: dimensions.fontLetterSpacingBody2,
    wordSpacing: dimensions.fontWordSpacingBody2,
  );

  TextStyle get button => _button ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeButton,
    height: dimensions.fontHeightButton,
    fontWeight: FontWeight.w500,
    letterSpacing: dimensions.fontLetterSpacingButton,
    wordSpacing: dimensions.fontWordSpacingButton,
  );

  TextStyle get caption => _caption ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeCaption,
    height: dimensions.fontHeightCaption,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingCaption,
    wordSpacing: dimensions.fontWordSpacingCaption,
  );

  TextStyle get overline => _overline ??= _baseStyle.copyWith(
    inherit: false,
    color: colors.textPrimary,
    fontSize: dimensions.fontSizeOverline,
    height: dimensions.fontHeightOverline,
    fontWeight: FontWeight.w400,
    letterSpacing: dimensions.fontLetterSpacingOverline,
    wordSpacing: dimensions.fontWordSpacingOverline,
  );
}
